
/////////////////////////////////////////////////////////
//                                                     //
//            Bold EasyTable DataBase Adapter          //
//           Copyright (c) 2004, 2006 Borland, Grikon  //
//                                                     //
/////////////////////////////////////////////////////////

{ Global compiler directives }

{$include bold.inc}

unit BoldETInterfaces;

                                   INTERFACE
uses
  Classes,
  Db, EasyTable,
  BoldSQLDatabaseConfig,
  BoldDBInterfaces;
type
  { forward declarations }

  TBoldETParameter = class;
  TBoldETDatabase = class;
  TBoldETQuery = class;
  TBoldETQueryClass = class of TBoldETQuery;


  { TBoldETParameter }
  TBoldETParameter = class(TBoldParameterWrapper, IBoldParameter)
  private
    fParameter: TParam;
    function GetAsVariant: Variant;
    procedure SetAsVariant(const NewValue: Variant);
    function GetName: String;
    procedure Clear;
    function GetDataType: TFieldType;
    procedure SetDataType(Value: TFieldType);
    function GetAsBCD: Currency;
    function GetAsBoolean: Boolean;
    function GetAsDateTime: TDateTime;
    function GetAsCurrency: Currency;
    function GetAsFloat: Double;
    function GetAsInteger: Longint;
    function GetAsMemo: string;
    function GetAsString: string;
    function GetIsNull: Boolean;
    procedure SetAsBCD(const Value: Currency);
    procedure SetAsBlob(const Value: TBlobData);
    procedure SetAsBoolean(Value: Boolean);
    procedure SetAsCurrency(const Value: Currency);
    procedure SetAsDate(const Value: TDateTime);
    procedure SetAsDateTime(const Value: TDateTime);
    procedure SetAsFloat(const Value: Double);
    procedure SetAsInteger(Value: Longint);
    procedure SetAsMemo(const Value: string);
    procedure SetAsString(const Value: string);
    procedure SetAsSmallInt(Value: LongInt);
    procedure SetAsTime(const Value: TDateTime);
    procedure SetAsWord(Value: LongInt);
    procedure SetText(const Value: string);
    function GetParameter: TParam;
    procedure AssignFieldValue(source: IBoldField);
    property Parameter: TParam read GetParameter;
  public
    constructor create(ETParameter: TParam; DatasetWrapper: TBoldDatasetWrapper);
  end;
 
  { TBoldETQuery }
  TBoldETQuery = class(TBoldDataSetWrapper, IBoldQuery, IBoldExecQuery, IBoldParameterized)
  private
    FQuery: TEasyQuery;
    function GetParamCount: integer;
    function GetParams(i: integer): IBoldParameter;
    function GetQuery: TEasyQuery;
    function GetRecordCount: integer;
    function GetRequestLiveQuery: Boolean;
    procedure ClearParams;
    function GetRowsAffected: integer;
    function GetSQLText: String;
    function ParamByName(const Value: string): IBoldParameter;
    procedure AssignParams(Sourceparams: TParams);
    procedure AssignSQL(SQL: TStrings);
    procedure AssignSQLText(SQL: String);
    procedure SetRequestLiveQuery(NewValue: Boolean);
    function Createparam(FldType: TFieldType; const ParamName: string; ParamType: TParamType; Size: integer): IBoldParameter;
  protected
    function GetDataSet: TDataSet; override;
    procedure EndSQLBatch; virtual;
    procedure ExecSQL;
    procedure FailSQLBatch; virtual;
    procedure Open; override;
    procedure StartSQLBatch; virtual;
    property Query: TEasyQuery read GetQuery;
  public
    constructor Create(Query: TEasyQuery; DatabaseWrapper: TBoldDatabaseWrapper);
  end;

  { TBoldETDataBase }
  TBoldETDatabase = class(TBolddatabaseWrapper, IBoldDataBase)
  private
    fInTransaction: Boolean;
    FDatabase: TEasyDatabase;
    FCachedQuery: TEasyQuery;
    function GetConnected: Boolean;
    function GetDataBase: TEasyDatabase;
    function GetInTransaction: Boolean;
    function GetIsSQLBased: Boolean;
    function GetKeepConnection: Boolean;
    function GetLogInPrompt: Boolean;
    function SupportsTableCreation: boolean;
    procedure Close;
    procedure Commit;
    procedure Open;
    function GetTable: IBoldTable;
    procedure ReleaseTable(var Table: IBoldTable);
    procedure Rollback;
    procedure SetKeepConnection(NewValue: Boolean);
    procedure SetlogInPrompt(NewValue: Boolean);
    procedure StartTransaction;
    property Database: TEasyDatabase read GetDatabase;
    procedure ReleaseCachedObjects;
  protected
    procedure AllTableNames(Pattern: String; ShowSystemTables: Boolean; TableNameList: TStrings); override;
    function GetQuery: IBoldQuery; override;
    procedure ReleaseQuery(var Query: IBoldQuery); override;
  public
    constructor create(Database: TEasyDatabase; SQLDataBaseConfig: TBoldSQLDatabaseConfig);
    destructor destroy; override;
  end;

var
  BoldETQueryClass: TBoldETQueryClass = TBoldETQuery;

                                  IMPLEMENTATION

uses
  SysUtils, StrUtils,
  BoldRev,
  BoldUtils,
  dialogs,
  variants, ETSupport;

{ TBoldETQuery }
//----------------------------------------------------------------------------------------
procedure TBoldETQuery.AssignParams(Sourceparams: tparams);
var
  i: integer;
  Opar : TParam;
begin
  Query.Params.Clear;
  if assigned(SourceParams) and (SourceParams.Count > 0) then
  begin
    for i := 0 to Sourceparams.Count - 1 do with OPar do
    begin
      OPar:=Query.Params.CreateParam( Sourceparams[i].DataType,Sourceparams[i].Name, ptInput) as TParam;
      Size:=0;
      Value:=SourceParams[i].Value;
    end;
  end;
end;
//----------------------------------------------------------------------------------------
procedure TBoldETParameter.Clear;
begin
  
end;
//----------------------------------------------------------------------------------------
procedure TBoldETQuery.AssignSQL(SQL: TStrings);
begin
  Query.SQL.BeginUpdate;
  Query.SQL.Assign(SQL);
  Query.SQL.EndUpdate;
end;
//----------------------------------------------------------------------------------------
procedure TBoldETQuery.AssignSQLText(SQL: String);
begin
  Query.SQL.BeginUpdate;
  Query.SQL.Clear;
  BoldAppendToStrings(Query.SQL, SQL, true);
  Query.SQL.EndUpdate;
end;
//----------------------------------------------------------------------------------------
procedure TBoldETQuery.ClearParams;
begin
  Query.Params.Clear;
end;
//----------------------------------------------------------------------------------------
constructor TBoldETQuery.Create(Query: TEasyQuery; DatabaseWrapper: TBoldDatabaseWrapper);
begin
  inherited create(DatabaseWrapper);
  FQuery := Query;
end;
//----------------------------------------------------------------------------------------
function TBoldETQuery.CreateParam(FldType: TFieldType; const ParamName: string; ParamType: TParamType; Size: integer): IBoldParameter;
const
  ParamDir: Array[TParamType] of TParamType
           =(ptUnknown, ptInput, ptOutput, ptInputOutput,ptResult);
var MyPar : TParam;
begin
  MyPar:=Query.Params.CreateParam(fldType,ParamName,ParamDir[ParamType]) as TParam;
  result := TBoldETParameter.Create(MyPar, self);
end;
//----------------------------------------------------------------------------------------
procedure TBoldETQuery.EndSQLBatch;
begin
end;
//----------------------------------------------------------------------------------------
procedure TBoldETQuery.ExecSQL;
begin
  Query.SQL.Text := PrepareETQuery(Query.SQL.Text);
  try
    Query.ExecSQL;
  except
    on E: Exception do MessageDlg(E.Message + #13#10 +Query.SQL.Text, mtError, [mbOk], 0);
    end;
  end;
//----------------------------------------------------------------------------------------
procedure TBoldETQuery.FailSQLBatch;
begin
end;
//----------------------------------------------------------------------------------------
function TBoldETQuery.GetDataSet: TDataSet;
begin
  result := Query;
end;
//----------------------------------------------------------------------------------------
function TBoldETQuery.GetParamCount: integer;
begin
  result := Query.Params.count;
end;
//----------------------------------------------------------------------------------------
function TBoldETQuery.GetParams(i: integer): IBoldParameter;
begin
  result := TBoldETParameter.Create(Query.Params[i], self);
end;
//----------------------------------------------------------------------------------------
function TBoldETQuery.GetQuery: TEasyQuery;
begin
  result := FQuery;
end;
//----------------------------------------------------------------------------------------
function TBoldETQuery.GetRecordCount: integer;
begin
  Result := Query.RecordCount;
end;
//----------------------------------------------------------------------------------------
function TBoldETQuery.GetRequestLiveQuery: Boolean;
begin
  result := Query.RequestLive;
end;
//----------------------------------------------------------------------------------------
function TBoldETQuery.GetRowsAffected: integer;
begin
  result := Query.RowsAffected;
end;
//----------------------------------------------------------------------------------------
function TBoldETQuery.GetSQLText: String;
begin
  result := Query.SQL.Text;
end;
//----------------------------------------------------------------------------------------
procedure TBoldETQuery.Open;
var
  st:string;
  lskob,rskob,ltpos, badpos: integer;

begin
  st := Query.SQL.Text;
  st := LowerCase(st);
  repeat
    badpos :=Pos(#13, st);
    if badpos > 0 then
    begin
      System.Delete(st, badpos, 1);
      System.Insert(' ', st, badpos);
    end;
    badpos :=Pos(#10, st);
    if badpos > 0 then
    begin
      System.Delete(st, badpos, 1);
      System.Insert(' ', st, badpos);
    end;
  until badpos = 0;
  //  linktable_alias     
  ltpos :=Pos('linktable_alias', st);
  if ltpos > 0 then
  begin
    //     
    lskob := PosEx('(', st, ltpos-1);
    if lskob > 0 then
    begin
      //   
      System.Delete(st, lskob, 1);
      System.Insert(' ', st, lskob);
     //   
      rskob := PosEx(')', st, ltpos+2);
      System.Delete(st, rskob, 1);
      System.Insert(' ', st, rskob);
    end;
  end;
  Query.SQL.Text := st;
  BoldLogSQL(Query.SQL);
  inherited;
end;
//----------------------------------------------------------------------------------------
function TBoldETQuery.ParamByName(const Value: string): IBoldParameter;
var
  Param: TParam;
begin
  Param := Query.ParamByName(Value);
  if assigned(Param) then result := TBoldETParameter.Create(Param, self)
  else result := nil;
end;
//----------------------------------------------------------------------------------------
procedure TBoldETQuery.SetRequestLiveQuery(NewValue: Boolean);
begin
  Query.RequestLive := NewValue;
end;
//----------------------------------------------------------------------------------------
procedure TBoldETQuery.StartSQLBatch;
begin
end;
//----------------------------------------------------------------------------------------
{ TBoldETDataBase }
//----------------------------------------------------------------------------------------
procedure TBoldETDatabase.AllTableNames(Pattern: String; ShowSystemTables: Boolean; TableNameList: TStrings);
begin
  Database.GetTablesList(TableNameList);
end;
//----------------------------------------------------------------------------------------
procedure TBoldETDatabase.Close;
begin
  Database.Close;
end;
//----------------------------------------------------------------------------------------
procedure TBoldETDatabase.Commit;
begin
  fInTransaction := False;
//  Database.Commit;
end;
//----------------------------------------------------------------------------------------
constructor TBoldETDatabase.create(DataBase: TEasyDatabase; SQLDataBaseConfig: TBoldSQLDatabaseConfig);
begin
  inherited create(SQLDataBaseConfig);
  FDataBase := DataBase;
end;
//----------------------------------------------------------------------------------------
destructor TBoldETDatabase.destroy;
begin
  inherited;
  FDatabase := nil;
  FreeAndNil(fCachedQuery);
end;
//----------------------------------------------------------------------------------------
function TBoldETDatabase.GetConnected: Boolean;
begin
  result := Database.Connected;
end;
//----------------------------------------------------------------------------------------
function TBoldETDatabase.GetDataBase: TEasyDatabase;
begin
  result := FDataBase;
end;
//----------------------------------------------------------------------------------------
function TBoldETDatabase.GetInTransaction: Boolean;
begin
  result := fInTransaction;
end;
//----------------------------------------------------------------------------------------
function TBoldETDatabase.GetIsSQLBased: Boolean;
begin
  result := true;
end;
//----------------------------------------------------------------------------------------
function TBoldETDatabase.GetKeepConnection: Boolean;
begin
  result := true;
end;
//----------------------------------------------------------------------------------------
function TBoldETDatabase.GetLogInPrompt: Boolean;
begin
                                      Result := True;
//  result := dataBase.LoginPrompt;
end;
//----------------------------------------------------------------------------------------
function TBoldETDatabase.GetQuery: IBoldQuery;
var
  Query: TEasyQuery;
begin
  if assigned(fCachedQuery) then
  begin
    Query := fCachedQuery;
    fCachedQuery := nil;
  end
  else
  begin
    Query := TEasyQuery.Create(nil);
    Query.DatabaseName := Database.DatabaseName;
  end;
  result := BoldETQueryClass.Create(Query, self);
end;
//----------------------------------------------------------------------------------------
function TBoldETDatabase.GetTable: IBoldTable;
begin
  result := nil;
end;
//----------------------------------------------------------------------------------------
procedure TBoldETDatabase.Open;
begin
  Database.Open;
end;
//----------------------------------------------------------------------------------------
procedure TBoldETDatabase.ReleaseCachedObjects;
begin
  FreeAndNil(fCachedQuery);
end;
//----------------------------------------------------------------------------------------
procedure TBoldETDatabase.ReleaseQuery(var Query: IBoldQuery);
var
  oquery: TBoldETQuery;
begin
  if Query.Implementor is TBoldETQuery then begin
    oquery := Query.Implementor as TBoldETQuery;
    Query := nil;
    if not assigned(FCachedQuery) then begin
      FCachedQuery := oquery.FQuery;
      if FCachedQuery.Active then
        FCachedQuery.Close;
      FCachedQuery.SQL.Clear;
    end
    else
      oquery.FQuery.free;
    oquery.Free;
  end;
end;
//----------------------------------------------------------------------------------------
procedure TBoldETDatabase.ReleaseTable(var Table: IBoldTable);
begin
  raise Exception.Create('BoldETAdapter.ReleaseTable: Operation not supported');
end;
//----------------------------------------------------------------------------------------
procedure TBoldETDatabase.Rollback;
begin
  fInTransaction := False;
end;
//----------------------------------------------------------------------------------------
procedure TBoldETDatabase.SetKeepConnection(NewValue: Boolean);
begin
end;
//----------------------------------------------------------------------------------------
procedure TBoldETDatabase.SetlogInPrompt(NewValue: Boolean);
begin
//  Database.LoginPrompt := NewValue;
end;
//----------------------------------------------------------------------------------------
procedure TBoldETDatabase.StartTransaction;
begin
  fInTransaction := True;
end;
//----------------------------------------------------------------------------------------
function TBoldETDatabase.SupportsTableCreation: boolean;
begin
                                        result := true;
end;
//----------------------------------------------------------------------------------------
constructor TBoldETParameter.create(ETParameter: TParam; DatasetWrapper: TBoldDatasetWrapper);
begin
  inherited Create(DatasetWrapper);
  fParameter := ETParameter;
end;
//----------------------------------------------------------------------------------------
function TBoldETParameter.GetAsBCD: Currency;
begin
  result := parameter.Value;
end;
//----------------------------------------------------------------------------------------
function TBoldETParameter.GetAsBoolean: Boolean;
begin
  result := parameter.Value;
end;
//----------------------------------------------------------------------------------------
function TBoldETParameter.GetAsCurrency: Currency;
begin
  result := parameter.Value;
end;
//----------------------------------------------------------------------------------------
function TBoldETParameter.GetAsDateTime: TDateTime;
begin
  result := parameter.Value;
end;
//----------------------------------------------------------------------------------------
function TBoldETParameter.GetAsFloat: Double;
begin
  result := parameter.Value;
end;
//----------------------------------------------------------------------------------------
function TBoldETParameter.GetAsInteger: Longint;
begin
  result := parameter.Value;
end;
//----------------------------------------------------------------------------------------
function TBoldETParameter.GetAsMemo: string;
begin
  result := parameter.Value;
end;
//----------------------------------------------------------------------------------------
function TBoldETParameter.GetAsString: string;
begin
  result := parameter.Value;
  if result = DatasetWrapper.DatabaseWrapper.SQLDatabaseConfig.EmptyStringMarker then
  result := '';
end;
//----------------------------------------------------------------------------------------
function TBoldETParameter.GetAsVariant: Variant;
begin
  result := parameter.Value;
end;
//----------------------------------------------------------------------------------------
function TBoldETParameter.GetDataType: TFieldType;
begin
  result := parameter.DataType;
end;
//----------------------------------------------------------------------------------------
function TBoldETParameter.GetIsNull: Boolean;
begin
  result := VarIsNull(Parameter.value)
end;
//----------------------------------------------------------------------------------------
function TBoldETParameter.GetName: String;
begin
  result := Parameter.Name;
end;
//----------------------------------------------------------------------------------------
function TBoldETParameter.GetParameter: TParam;
begin
  result := fParameter;
end;
//----------------------------------------------------------------------------------------
procedure TBoldETParameter.SetAsBCD(const Value: Currency);
begin
  Parameter.Value := Value;
end;
//----------------------------------------------------------------------------------------
procedure TBoldETParameter.SetAsBlob(const Value: TBlobData);
begin
  with Parameter do begin
  if (DataType = ftUnknown) or (DataType = ftBlob)
  then  begin
          DataType := ftBlob;
          ParamType:=ptInput;
        end;
  end;
  if value = '' then
    Parameter.Value := DatasetWrapper.DatabaseWrapper.SQLDatabaseConfig.EmptyStringMarker
  else
    Parameter.Value := Value;
end;
//----------------------------------------------------------------------------------------
procedure TBoldETParameter.SetAsBoolean(Value: Boolean);
begin
  Parameter.Value := Value;
end;
//----------------------------------------------------------------------------------------
procedure TBoldETParameter.SetAsCurrency(const Value: Currency);
begin
  Parameter.Value := Value;
end;
//----------------------------------------------------------------------------------------
procedure TBoldETParameter.SetAsDate(const Value: TDateTime);
begin
  Parameter.Value := Value;
end;
//----------------------------------------------------------------------------------------
procedure TBoldETParameter.SetAsDateTime(const Value: TDateTime);
begin
  Parameter.Value := Value;
end;
//----------------------------------------------------------------------------------------
procedure TBoldETParameter.SetAsFloat(const Value: Double);
begin
  Parameter.Value := Value;
end;
//----------------------------------------------------------------------------------------
procedure TBoldETParameter.SetAsInteger(Value: Integer);
begin
  Parameter.Value := Value;
end;
//----------------------------------------------------------------------------------------
procedure TBoldETParameter.SetAsMemo(const Value: string);
begin
  Parameter.Value := Value;
end;
//----------------------------------------------------------------------------------------
procedure TBoldETParameter.SetAsSmallInt(Value: Integer);
begin
  Parameter.Value := Value;
end;
//----------------------------------------------------------------------------------------
procedure TBoldETParameter.SetAsString(const Value: string);
begin
  if value = '' then
    Parameter.Value := DatasetWrapper.DatabaseWrapper.SQLDatabaseConfig.EmptyStringMarker
  else
    Parameter.Value := Value
end;
//----------------------------------------------------------------------------------------
procedure TBoldETParameter.SetAsTime(const Value: TDateTime);
begin
  Parameter.Value := Value;
end;
//----------------------------------------------------------------------------------------
procedure TBoldETParameter.SetAsVariant(const NewValue: Variant);
begin
  Parameter.Value := NewValue;
end;
//----------------------------------------------------------------------------------------
procedure TBoldETParameter.SetAsWord(Value: Integer);
begin
  Parameter.Value := Value;
end;
//----------------------------------------------------------------------------------------
procedure TBoldETParameter.SetDataType(Value: TFieldType);
begin
  Parameter.DataType := Value;
end;
//----------------------------------------------------------------------------------------
procedure TBoldETParameter.SetText(const Value: string);
begin
  Parameter.Value := Value;
end;
//----------------------------------------------------------------------------------------
procedure TBoldETParameter.AssignFieldValue(source: IBoldField);
begin
  Parameter.Assign(Source.Field);
end;
//****************************************************************************************
end.